#include <fstream>
#include <iostream>
#include <iomanip>
#include <string>
#include <vector>

#include <productiondatabaseclient.h>
#include <boost/program_options.hpp>
#include "toolbox.cpp"

using namespace std;
using namespace ProductionDatabase;
namespace po = boost::program_options;

static bool irradiatedAPDs = false;
static string fileName = "serials.dat";
static uint boxNo = 0;
static bool debug = false;
static string username = "";
static string password = "";

void processArgumentsAndQueryMissing(int m_argc, char* m_argv[]);

int main(int argc, char* argv[]) {
    ProductionDatabaseClient *proddb = new ProductionDatabaseClient();
    processArgumentsAndQueryMissing(argc, argv);

    if (debug) cout << "Intitialized apdBatchSetter with version " << proddb->getVersion() << " of the database access libraries." << endl << endl
                    << "Now trying to set Batch number " << boxNo << " for ADPs from serial file " << fileName << endl;

    vector<string> apdSerials;

    apdSerials = loadSerialsFromFileName(fileName);

    if (username == "" || password == "") proddb->queryCredentials();
    else proddb->setCredentials(username, password);
    DatabaseClientResponse response = proddb->checkConnectivityAndCredentials();
    if (response != Successful ) {
        cerr << "Connection to database failed because of error: " << response << endl;
        return response;
    }
    if (debug) cout << "Connection successful!" << endl;

    cout << "\nBox " << boxNo << " was not assigned to APDs from serial file " << fileName << "because the corresponding field does not exist yet."
         << "\nThank you for using apdBoxSetter! :)" << endl << endl;

    return (-apdSerials.empty());
}

void processArgumentsAndQueryMissing(int m_argc, char* m_argv[]) {
    po::options_description desc("Available options");
    desc.add_options()
            ("help", "produce help message")
            ("type", po::value<string>(), "(required) type of APDs [new, irr]")
            ("box", po::value<int>(), "(required) box number to be assigned")
            ("fileName", po::value<string>(), "file name or serial numbers [serials.dat]")
            ("debug", "emit additional messages for debugging purposes")
            ("user", po::value<string>(), "User name to connect to DB. Only used when combined with pass!")
            ("pass", po::value<string>(), "Password to connect to DB. Only used when combined with user!")
            ;
    po::variables_map vm;
    po::store(po::parse_command_line(m_argc, m_argv, desc), vm);
    po::notify(vm);

    if (vm.count("help")) {
        cout << desc << "\n";
        exit(0);
    }
    if (vm.count("type")) {
        if (vm["type"].as<string>() == "new") {
            irradiatedAPDs = false;
        }
        else if (vm["type"].as<string>() == "irr") {
            irradiatedAPDs = true;
        } else {
            cerr << "Invalid type! Possible APD types: 'new' or 'irr'" << endl;
            exit(0);
        }
    } else {
        string temp = "";
        cout << "Have these APDs been irradiated? (yes / no)" << endl;
        cin >> temp;
        cout << endl;
        if (temp == "yes" || temp == "y") {
            irradiatedAPDs = true;
        }
        else if (temp == "no" || temp == "n") {
            irradiatedAPDs = false;
        }
        else {
            cerr << "Come on, you can't even give me a straight answer on that one? I'm sick of this. I'm leaving. Try again when you have the information you need." << endl;
            exit(0);
        }
    }
    if (vm.count("box")) {
        boxNo = uint(vm["batch"].as<int>());
    } else {
        cout << "Which box should these APDs be assigned to?" << endl;
        try {
            cin >> boxNo;
            cout << endl;
        } catch (...) {
            boxNo = 0;
        }
        if (boxNo <= 0) {
            cerr << "Invalid batch number!" << endl;
            exit(-1);
        }
    }
    if (vm.count("fileName")) {
        fileName = vm["fileName"].as<string>();
        cout << "Reading serials from " << fileName << endl;
    }
    if (vm.count("user")) {
        username = vm["user"].as<string>();
    }
    if (vm.count("pass")) {
        password = vm["pass"].as<string>();
    }
    if (vm.count("debug")) {
        debug = true;
    }
}
